package Net::Bummskraut::Listener;
use strict;
use Socket;
use AnyEvent;
use IO::Socket::INET;
use Net::Bummskraut::Connection;
use Net::Bummskraut::Event;

our @ISA = qw/Net::Bummskraut::Event/;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {@_};
   bless $self, $class;
   $self->start_listener;
   return $self;
}

sub start_listener {
   my ($self) = @_;

   my $sock = IO::Socket::INET->new(
         Listen    => 5,
         ReuseAddr => 1,
         Reuse     => 1,
         LocalPort => $self->{port} || 16100,
         Proto     => 'tcp'
   );

   $sock or die "Couldn't create listener: $!";

   $self->{listener} =
      AnyEvent->io (poll => 'r', fh => $sock, cb => sub {
         my $cl = $sock->accept ()
            or die "couldn't accept client: $!";
         $cl->autoflush (1);
         $self->handle_client ($cl);
      });
}

sub foreach_client {
   my ($self, $cb, @arg) = @_;
   for (values %{$self->{clients}}) {
      $cb->($self, $_, @arg);
   }
}

sub foreach_client_except {
   my ($self, $ex, $cb, @arg) = @_;
   for (grep { $ex ne $_ } values %{$self->{clients}}) {
      $cb->($self, $_, @arg);
   }
}

sub handle_client {
   my ($self, $cl) = @_;
   my ($chost, $cport) = ($cl->peerhost (), $cl->peerport ());
   my $lid = "$chost:$cport";

   my $cl = $self->{clients}->{$cl} =
      Net::Bummskraut::Connection->new (
         socket => $cl, host => $chost, port => $cport,
         event => {
            disconnect => sub {
               $self->event (disconnect => $cl);
               delete $self->{clients}->{$cl};
            }
         }
      );

   $self->event (connect => $cl);
   $cl->start_reader;
   $cl->start_writer;
}

1;
