package Net::Bummskraut::Scheme;
use strict;
use Net::Bummskraut::Event;

our @ISA = qw/Net::Bummskraut::Event/;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {@_};
   bless $self, $class;
   return $self;
}

sub unsubscribe {
   my ($self, $id, $cb) = @_;
   $cb->(0, "operation not supported");
}

sub subscribe {
   my ($self, $id, $cb) = @_;
   $cb->(0, "operation not supported");
}

sub subscriptions { () }

sub get_src { "void:none" }

sub raw { }

sub meta { }

sub cfg_keys { () }

sub cfg_get {
   my ($self, $key) = @_;

   if (my $v = $self->{cfg}->{$key}) {
      if (ref $v eq 'ARRAY') {
         return @$v;
      } elsif (ref $v eq 'HASH') {
         return map { "$_ => '$v->{$_}'" } keys %$v;
      } else {
         return $v;
      }
   }

   return undef;
}

sub cfg_set {
   my ($self, $key, $value, $remove) = @_;

   if (ref $self->{cfg}->{$key} eq 'ARRAY') {
      if ($remove) {
         @{$self->{cfg}->{$key}} =
            grep { $value ne $_ } @{$self->{cfg}->{$key}};
      } else {
         push @{$self->{cfg}->{$key}}, $value;
      }
      $self->event (cfg_change => $key);
      $self->{server}->commit_cfg;

   } elsif (ref $self->{cfg}->{$key} eq 'HASH') {
      my ($subkey, $val) = split /\s+/, $value;

      if ($remove) {
         delete $self->{cfg}->{$key}->{$subkey};
      } else {
         $self->{cfg}->{$key}->{$subkey} = $val;
      }

      $self->event (cfg_change => $key);
      $self->{server}->commit_cfg;

   } else {
      $self->{cfg}->{$key} = $remove ? '' : $value;

      $self->event (cfg_change => $key);
      $self->{server}->commit_cfg;
      return 1
   }

   0
}


sub reload { }

sub refresh { }

sub message { }
sub meta { }
sub raw { }

sub set_group {
   my ($self, $id, $group, $cb) = @_;
   $cb->(0, 'not implemented');
}

sub set_nick {
   my ($self, $id, $nick, $cb) = @_;
   $cb->(0, 'not implemented');
}

sub set_presence {
   my ($self, $show, $state) = @_;
}

1;
