package Net::Bummskraut::Scheme::Config;
use strict;
use Net::Bummskraut::Scheme;
use Data::Dumper;
our @ISA = qw/Net::Bummskraut::Scheme/;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {@_};
   bless $self, $class;

   $self->{server}->reg_scheme (cfg => $self);
   $self->{server}->change_state ("cfg:perl",   'present', 'available', 'perl');
   $self->{server}->change_state ("cfg:client", 'present', 'available', 'client');

   my %schemes = $self->{server}->get_schemes;

   for my $sc (grep { $_ ne 'cfg' } keys %schemes) {
      $self->{server}->change_state ("cfg:$sc",
         'present', 'available', $sc.'-scheme-configurator');
      $self->{server}->add_subids ("cfg:$sc",
         (map { ["cfg:$sc|$_->[0]", "$_->[0]_key"] }
            $schemes{$sc}->cfg_keys),
         [ "cfg:$sc-console", "console" ],
         [ "cfg:client", "client" ],
      );
   }

   return $self;
}

sub get_src {
   my ($self, $dest) = @_;
   "cfg:client"
}

my $maxdepth = 2;

sub message {
   my ($self, $dest, $scope, $type, $content) = @_;

   my @msg;
   my %schemes = $self->{server}->get_schemes;
   my $act;
   my $src;

   if ($dest eq 'cfg:perl') {
      $self->{server}->distribute_message (
         $self->get_src ($dest), $dest, 'public', 'normal', 1, 0, time, $content
      );

      my @msg;
      my $r = eval $content;
      if ($@) {
         push @msg, "exception: $@";
      } else {
         $Data::Dumper::Maxdepth = $maxdepth;
         if (not grep { ref ($r) eq $_ } qw/HASH SCALAR ARRAY CODE REF GLOB LVALUE/) {
            $r = Data::Dumper->Dump ([$r], [qw/result/]);
         } elsif (ref ($r) eq 'HASH' or ref ($r) eq 'ARRAY') {
            $r = Data::Dumper->Dump ([$r], [qw/result/]);
         }
         push @msg, $r;
      }

      for (@msg) {
         $self->_dm ($dest, $dest, $_);
      }
      return;

   } elsif ($dest =~ m/^cfg:(\S+)$/) {
      my $sc = $1;
      $src = "cfg:$sc-console";

      $self->{server}->distribute_message (
         $self->get_src ($dest), $dest, 'public', 'normal', 1, 0, time, $content
      );

      unless ($schemes{$sc}) {
         $self->_dm ($src, $dest, "No such scheme: '$sc'");
         return
      }

      if ($content =~ /^(\S+?)_key:\s*(.*)$/) {
         my $key = $1;
         my $msg = $2;
         my ($cmd, $arg) = split /\s+/, $msg, 2;
         $cmd = lc $cmd;

         $src = "cfg:$sc|$key";

         my $rel;

         if ($cmd eq 'set') {
            $rel = $schemes{$sc}->cfg_set ($key, $arg);
            push @msg, "set value ($arg)";

         } elsif ($cmd eq 'remove') {
            $rel = $schemes{$sc}->cfg_set ($key, $arg, 1);
            push @msg, "removed value" . ($arg ? "($arg)" : "")

         } elsif ($cmd eq 'get') {
         } else {
            push @msg, "Unrecognized command: $cmd"
         }

         push @msg, "= $_" for $schemes{$sc}->cfg_get ($key);
         push @msg, "(this config key needs a 'reload $sc')"
            if $rel;

      } else {
         my ($cmd, $arg) = split /\s+/, $content, 2;

         if ($cmd eq 'list') {
            for my $key (map { $_->[0] } $schemes{$sc}->cfg_keys) {
               my @vals = $schemes{$sc}->cfg_get ($key);
               my %desc = map { ($_->[0], $_->[1]) } $schemes{$sc}->cfg_keys ();

               $self->_dm ("cfg:$sc|$key", $dest, "-=[ $desc{$key} ]=-");
               for (@vals) {
                  $self->_dm ("cfg:$sc|$key", $dest, "= $_");
               }
            }

         } elsif ($cmd eq 'sups') {
            my @sub = $schemes{$sc}->subscriptions;
            push @msg, "Subscriptions:";
            for (@sub) {
               push @msg, "   $_";
            }

         } elsif ($cmd eq 'reload') {
            push @msg, "Reloading... this may take a while...";
            $act = sub { $schemes{$sc}->reload };

         } elsif ($cmd eq 'refresh') {
            push @msg, "Refreshing...";
            $act = sub { $schemes{$sc}->refresh };

         } elsif ($cmd eq 'help') {
            my @help = (
               [list    => "list     - Lists the whole configuration."],
               [reload  => "reload   - Reloads this scheme."],
               [refresh => "refresh  - Refreshes this scheme."],
               [sups    => "sups     - Lists subscriptions."],
            );

            for (@help) {
               push @msg, sprintf "%-10s: %s", $_->[0], $_->[1]
            }
         }
      }
   }

   $self->_dm ($src, $dest, @msg);

   $act->() if $act;
}

sub _dm {
   my ($self, $src, $dest, @msg) = @_;
   for (@msg) {
      $self->{server}->distribute_message (
         $src, $dest, 'public', 'normal', 0, 0, time, $_
      );
   }
}
1;
