package Net::Bummskraut::Scheme::XMPP::GUI;
use strict;
use Net::GUI;
use Net::GUI::Element qw/build_el/;
use Net::GUI::Event;
use Scalar::Util qw/weaken/;
use POSIX qw/strftime/;
use Net::XMPP2::Util qw/dump_twig_xml/;
use XML::Twig;

our @ISA = qw/Net::GUI::Event/;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {@_};
   bless $self, $class;
   $self->init;
   $self
}

sub init {
   my ($self) = @_;
   my $s = $self->{scheme};
   my $cl = $s->{xmpp};
   $cl->reg_cb (
      recv_stanza_xml => sub {
         my ($cl, $acc, $node) = @_;
         my $time = time;
         my $name = $node->name;
         if (defined $node->attr ('type')) {
            $name .= " [".$node->attr ('type')."]";
         }
         my $id = "?";
         if (defined $node->attr ('id')) {
            $id = $node->attr ('id')
         }
         my $entry = [
            'R', $time, $acc->bare_jid, $name, $id, dump_twig_xml ($node->as_string)
         ];
         push @{$self->{stanza_log}}, $entry;
         $self->event (stanza_log_add => $entry);
      },
      send_stanza_data => sub {
         my ($cl, $acc, $data) = @_;
         my $time = time;
         my $tw = XML::Twig->new;
         my $name = '?';
         my $id = '?';
         if ($tw->safe_parse ($data)) {
            $name = $tw->root->local_name;
            if (defined $tw->root->{att}->{type}) {
               $name .= " [".$tw->root->{att}->{type}."]";
            }
            if (defined $tw->root->{att}->{id}) {
               $id = $tw->root->{att}->{id};
            }
         }
         my $entry = [
            'S', $time, $acc->bare_jid, $name, $id, dump_twig_xml ($data)
         ];
         push @{$self->{stanza_log}}, $entry;
         $self->event (stanza_log_add => $entry);
      }
   );
}

sub start {
   my ($self, $con, $heap) = @_;

   weaken $con;
   weaken $heap;
   weaken $self;

   my $scheme = $self->{scheme};

   unless ($heap->{xmpp_win}) {
      my $pkt = $con->capture (sub {
         my $w = build_el ([
            window => { name => "xmpp-control" },
            [button => { label => 'messagelog' }, {
               clicked => sub { $self->open_messagelog ($con, $heap) }
            }],
            { closed => sub { delete $heap->{xmpp_win} } }
         ]);
         $heap->{xmpp_win} = $w;
         $w->show;
      });
      $con->send_commands ($pkt);
   }
}

sub open_messagelog {
   my ($self, $con, $heap) = @_;

   return if $heap->{msg_log};

   my $pkt = $con->capture (sub {
      my $text = build_el (['text', { fill => 0 } ]);
      my $list = $heap->{msg_log_list} = build_el ([
         list => {
            fields => [ Dir => 'string', Time => 'string',
                        Type => 'string', ID => 'string', Account => 'string' ],
            expand => 1,
         }, {
            row_activated => sub {
               my ($e, $row) = @_;
               my $txt;
               for my $offs (1..5) {
                  my $i = ($row->[5] - 1) + $offs;
                  next unless $i >= 0;
                  my $ent = $self->{stanza_log}->[$i];
                  last unless defined $ent;
                  my $time = strftime ("%T %F %z", localtime ($ent->[1]));
                  $txt .= "$ent->[0] ($time)[$ent->[2]]:\n$ent->[5]\n"
                          .("-" x 80)."\n";
               }
               $text->set (text => $txt);
            }
         }
      ]);
      my $i = 0;
      for my $l (@{$self->{stanza_log}}) {
         $list->set (append_row => [ $l->[0], $l->[1], $l->[3], $l->[4], $l->[2], $i++ ]);
      }
      $self->reg_cb (stanza_log_add => sub {
         my ($self, $l) = @_;
         $list->set (append_row => [ $l->[0], $l->[1], $l->[3], $l->[4], $l->[2], $i++ ]);
      });
      my $w = build_el ([
         window => { name => 'xmpp-msg-log' },
         { closed => sub { delete $heap->{msg_log} } },
         [pane => { dir => 'v' },
            $list,
            $text
         ]
      ]);
      $heap->{msg_log} = $w;
      $w->show;
   });
   $con->send_commands ($pkt);
}

1
