package Net::Bummskraut::Util;
use Exporter;
use JSON::XS;

our @ISA = qw/Exporter/;

our @EXPORT = qw/load_cfg save_cfg split_irc_uri join_irc_uri/;

sub load_cfg {
   my $file = shift;
   return unless -e $file;
   open my $cfgh, "<", $file or die "Couldn't open '$file': $!";
   $cfg = from_json (do { local $/; <$cfgh> });
}

sub save_cfg {
   my ($file, $cfg) = @_;
   open my $cfgh, ">", $file or die "Couldn't open for writing '$file': $!";
   print $cfgh (JSON::XS->new->canonical (1)->pretty (1)->encode ($cfg)); }

sub uri_escape {
   my ($in) = @_;
   $in =~ s/([^a-zA-Z0-9_.-])/uc(sprintf("%%%02x",ord($1)))/eg;
   $in
}

sub uri_unescape {
   my ($in) = @_;
   $in =~ s/%([0-9a-fA-F]{2})/chr hex ($1)/eg;
   $in
}

sub split_irc_uri {
   my ($uri) = @_;
   if ($uri =~  /^irc:([~@*!+&^\/])([^@]*)@(.+)$/) {
      my $p = $1 eq '*' ? '#' : $1;
      (($p ne '~' ? "$p" : '') . (uri_unescape $2), $3)
   } else {
      (undef, undef)
   }
}

sub join_irc_uri {
   my ($target, $auth) = @_;
   my $p;
   $target = '' unless defined $target;

   if ($target =~ m/^([#!+&])(.*)$/) {
      $p = $1 eq '#' ? '*' : $1;
      $target = $2;
   } else {
      $p = "~";
   }
   # mapped to urls IRC places are lower case
   $target = lc $target;
   $auth   = lc $auth;
   $target = uri_escape $target;
   "irc:$p$target\@$auth"
}
